"""
Development settings for server project.
"""

import os
import yaml
from django.utils.translation import ugettext as _
from django.utils.translation import gettext_lazy as _
import environ
import coloredlogs, logging
from .logging_config import setup_logging

# Utility function
def str_to_bool(value):
    logger.debug(f'str to bool {value} {type(value).__name__}')
    if isinstance(value, bool):
        logger.verbose(f'{value} is already bool')
        return value
    if isinstance(value, str):
        value = value.strip()
        bool_value = value.lower() in {'True','true', '1', 'yes', 'y',True,'True'}
        logger.debug(f'{value.lower()} string : bool result {bool_value}')
        return bool_value
    else:
        error_message = f"str_to_bool function :  {type(value).__name__} type not handled"
        if DEBUG:
            raise SystemExit(logger.critical(error_message))
        else:
            logger.critical(error_message)

# Setup logging
logger = setup_logging()

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Initialize environment variables
env = environ.Env(
    # Set casting, default value
    DEBUG=(bool, False)
)
env_file_path = os.path.join(BASE_DIR, '.env')
environ.Env.read_env(env_file_path)


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '{levelname} {asctime} {module} {message}',
            'style': '{',
        },
        'detailed': {
            'format': '{levelname} {asctime} {module} {message} {exc_info}',
            'style': '{',
        },
        'simple': {
            'format': '{levelname} {message}',
            'style': '{',
        },
    },
    'handlers': {
        'console': {
            'level': 'INFO',  # Capture DEBUG and higher level logs
            'class': 'logging.StreamHandler',
            # No formatter specified, so default format will be used
        },
        'file': {
            'level': 'ERROR',  # Capture ERROR and higher level logs
            'class': 'logging.FileHandler',
            'filename': os.path.join(BASE_DIR, 'server/django_errors.log'),
            'formatter': 'detailed',
            # No formatter specified, so default format will be used
        },
    },
    'loggers': {
        'django': {
            'handlers': ['console', 'file'],  # Attach both handlers
            'level': 'INFO',  # Minimum level for this logger
            'propagate': False,  # Prevents propagation to root logger
        },
        'django_q': {  # Logger for Django-Q
            'handlers': ['console', 'file'],  # Same handlers as Django logs
            'level': 'INFO',  # Adjust as needed
            'propagate': False,  # Prevent propagation to the root logger if needed
        },
    },
}
#######
### Set .env Variables - except email
#######
DEBUG = str_to_bool(env('DEBUG'))
TEAM_SIGNATURE = env('TEAM_SIGNATURE')
INSTANCE_MAINTAINER = env('INSTANCE_MAINTAINER')

def djangoldp_config():
    """Get DjangoLDP parameters from configuration file"""
    path = os.path.join(BASE_DIR, 'packages.yml')
    with open(path, 'r') as f:
        content = yaml.safe_load(f)
        config = content.get('server', {})
        try:
            config['packages'] = list(content.get('ldppackages').keys())
        except AttributeError:
            config['packages'] = []
        return config

DJANGOLDP_PACKAGES = djangoldp_config().get('packages', [])

####################################################
##########  Site Settings
####################################################
SECRET_KEY = 'wr1tfli=&(oervg-g8_6*zlrnny=yha^+smg$zq@h@#9lmi%p2'
### Localhost setup ###
if 'localhost' in djangoldp_config().get('site_url'):
    ALLOWED_HOSTS = ['localhost']

### Production Setup ###
# Add the domain name of all your hosts below
# By default, the site url in packages.yml is added
else:    
    ALLOWED_HOSTS = [djangoldp_config().get('site_url').replace('https://','')]

# DOMAINE NAME
# By default, the site url in packages.yml is used
# if you're using a sub-domain, make sure to include it ('https://subdomain.example.org')
INSTANCE_DOMAINE_NAME = djangoldp_config().get('site_url')
# It will be used to :
# - Build the shortlinks of your actions
# - Save the emails sent to reprensentatives to your database (important for federation)

# ORGANISATION NAME
# Add the name of your organisation beflow
# It will be displayed next to your action on other instances
INSTANCE_NAME = env('INSTANCE_NAME')

# FEDERATION
# Make 'True' to activate federation and display changes on the template 
FEDERATION_ACTIVE = str_to_bool(env('FEDERATION_ACTIVE'))


# to depreciate
CAMPAIGN_FINANCING = False

#####
## Display / Hide site sections
####
GLOBAL_ACTION=str_to_bool(env('GLOBAL_ACTION')) # la section 'Action' - Interpellation / map de mobilisation
PUBLIC_ENQUIRIES=str_to_bool(env('PUBLIC_ENQUIRIES')) # la section "Enquête publique" 
SUPPORT_CALLS=str_to_bool(env('SUPPORT_CALLS')) # La section "Appel à soutiens"
REPRESENETATIVE_DATABASE=str_to_bool(env('REPRESENETATIVE_DATABASE')) # Afficher la page listant tous les élues
PLUGIN_LAW_TRACKER=str_to_bool(env('PLUGIN_LAW_TRACKER')) # Suivi des lois au parlement, exemple ici https://agir.risefor.org/law-tracker/


from startupinfo import printStartupInfo
printStartupInfo(INSTANCE_NAME)


############
# ACTIVATE HTTPS
############
# Uncomment 4 lines below to activate
 
#SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https')
#SECURE_SSL_REDIRECT = True
#SESSION_COOKIE_SECURE = True
#CSRF_COOKIE_SECURE = True


####################################################
##########  MULTI-LINGUAL TEMPLATE
####################################################
#Make True to activate international template
INTERNATIONAL_TEMPLATE = str_to_bool(env('INTERNATIONAL_TEMPLATE'))



#default languages 
# For Non international : must also be changed in 'else' below
LANGUAGE_CODE = 'fr'
MODELTRANSLATION_DEFAULT_LANGUAGE = 'fr'
TIME_ZONE = 'UTC'
#add country prefix by default, make True to active
prefix_default_language = False

if INTERNATIONAL_TEMPLATE == True:
    #Add wanted languages below
    LANGUAGES = (
        ('en', ('English-Global')),
        ('us', ('English-USA')),
        ('fr', ('French')),
        ('es', ('Spanish')),
    )
    COUNTRIES_OVERRIDE = {
        'Global': {
            'name': _('Global'),
            'code':_('en')
        },
    }
else: 
    LANGUAGES = (
        #change default language below
        ('fr', ('French')),
    )
#for django, don't touch
USE_I18N = True
USE_L10N = True
#USE_TZ = True # setting this true creates assertion error
COUNTRIES_FLAG_URL = '/static/flags/img/{code}.svg'
LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale'),
)

####################################################
##########  Application definition
####################################################

WSGI_APPLICATION = 'server.wsgi.application'
INSTALLED_APPS = list(DJANGOLDP_PACKAGES)
INSTALLED_APPS.extend([
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sitemaps',
    'rest_framework',
    'djangoldp',
    'guardian',
    'reversion',
    'django_q',
    'django_admin_logs',
])

REST_FRAMEWORK = {
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
    ),
    'DEFAULT_PARSER_CLASSES': (
        'rest_framework.parsers.JSONParser',
    ),
}

ROOT_URLCONF = 'server.urls'
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'risefor_lobbying.context_processors.instanceInformation',
            ],
        },
    },
]

####################################################
##########  Database setup
####################################################
dataBaseType = env('dataBaseType').strip()
if dataBaseType == 'dev':
    # Database
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
            
        }
    }
#use other db
elif dataBaseType == 'postgresql':
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'NAME': env('DATABASE_NAME'),
            'USER': env('DATABASE_USER'),
            'PASSWORD': env('DATABASE_PASSWORD'),
            'DATABASE_TIME_ZONE':env('DATABASE_TIME_ZONE'),
            'HOST': env('DATABASE_HOST'),
            'PORT': env('DATABASE_PORT'),
        }
    }
else:
    raise SystemExit(logger.critical(f"You must choose a valid database type. You can update this in your .env file"))
    

####################################################
##########  Authentification setup
####################################################

AUTH_PASSWORD_VALIDATORS = []
AUTH_USER_MODEL = 'djangoldp_account.LDPUser'
LOGIN_REDIRECT_URL = '/'

if 'djangoldp_account' in INSTALLED_APPS:
    LOGIN_URL = '/auth/login/'
    OIDC_USERINFO = 'djangoldp_account.settings.userinfo'
    OIDC_REGISTRATION_ENDPOINT_REQ_TOKEN = False
    OIDC_REGISTRATION_ENDPOINT_ALLOW_HTTP_ORIGIN = True
    OIDC_IDTOKEN_SUB_GENERATOR = 'djangoldp_account.settings.sub_generator'
    OIDC_IDTOKEN_EXPIRE = 60 * 60 * 1600
    AUTH_USER_MODEL = 'djangoldp_account.LDPUser'
    MIDDLEWARE = [
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        #if activate cache, put it after localMiddleaware
        'django.middleware.locale.LocaleMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
        'djangoldp_account.auth.middleware.JWTUserMiddleware'
    ]
    AUTHENTICATION_BACKENDS = [
        'djangoldp_account.auth.backends.EmailOrUsernameAuthBackend',
        'guardian.backends.ObjectPermissionBackend',
        'djangoldp_account.auth.backends.ExternalUserBackend'
    ]
else:
    MIDDLEWARE = [
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        #if activate cache, put it after localMiddleaware
        'django.middleware.locale.LocaleMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
    ]
    AUTHENTICATION_BACKENDS = ['django.contrib.auth.backends.ModelBackend', 'guardian.backends.ObjectPermissionBackend']

if 'djangoldp_notification' in INSTALLED_APPS:
    MIDDLEWARE.append('djangoldp_notification.middlewares.CurrentUserMiddleware')

USER_ANONYMOUS_PERMISSIONS=['add', 'view']
USER_AUTHENTICATED_PERMISSIONS=['view', 'add']
USER_OWNER_PERMISSIONS=['view', 'add', 'change']

GROUP_ANONYMOUS_PERMISSIONS=['view']
GROUP_AUTHENTICATED_PERMISSIONS=['view', 'add', 'change']
GROUP_OWNER_PERMISSIONS=['view', 'add', 'change']

ANONYMOUS_USER_NAME = None

####################################################
##########  Static files & Media 
####################################################
# Static files (CSS, JavaScript, Images)
APPEND_SLASH = False
SITE_URL = djangoldp_config().get('site_url') 
BASE_URL = SITE_URL
STATIC_URL = '/static/'
STATIC_DIR = os.path.join(BASE_DIR, 'static')
STATIC_ROOT = os.path.join(BASE_DIR, 'staticfiles')
STATICFILES_DIRS = [os.path.join(BASE_DIR, 'static'), ]
MEDIA_URL = '/media/'
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIAFILES_DIRS = [os.path.join(BASE_DIR, 'media'), ]
LDP_RDF_CONTEXT = 'https://cdn.happy-dev.fr/owl/hdcontext.jsonld'

# Chat
PROSODY_HTTP_URL = djangoldp_config().get('xmpp_url')
JABBER_DEFAULT_HOST = djangoldp_config().get('jabber_host')

####################################################
##########  Email Setup
####################################################
# Debug : Content of email will print in the console
# Production: Choose if you want to send using external SMTP sender or local postfix installation

DEFAULT_FROM_EMAIL = env('DEFAULT_FROM_EMAIL')
logger.debug(f"Emailing sending in .env {env('EMAIL_ACTIVATE_SENDING')}")
EMAIL_ACTIVATE_SENDING =  str_to_bool(env('EMAIL_ACTIVATE_SENDING'))
logger.debug(f"Email sending is {EMAIL_ACTIVATE_SENDING}")


logger.warning(f"Debug status : {DEBUG}")
logger.warning(f"Database used is : {dataBaseType}")
logger.warning(f"Emails will really send {EMAIL_ACTIVATE_SENDING}\n\n")
logger.info(f"Domaine name is set to {INSTANCE_DOMAINE_NAME}")
logger.info(f"Allowed hosts are {ALLOWED_HOSTS}\n\n")
logger.info(f"Instance Maintainer is : {INSTANCE_MAINTAINER}")
logger.info(f"Display name is : {INSTANCE_NAME}\n\n")
logger.info(f"Federation status set to {FEDERATION_ACTIVE}")
logger.info(f"International Template set to {INTERNATIONAL_TEMPLATE}\n\n")
logger.info(f"Default 'From' Email is {DEFAULT_FROM_EMAIL}")



#### Local / debug ####
EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

if EMAIL_ACTIVATE_SENDING == True:
    logger.notice("Emails will be sent for real\n")
    EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
    EMAIL_SERVER_TYPE = env('EMAIL_SERVER_TYPE').strip()
    logger.notice(f'Server type is {EMAIL_SERVER_TYPE}')
    if EMAIL_SERVER_TYPE == 'postfix':
        ### Postfix settup
        EMAIL_HOST_USER = env('EMAIL_HOST_USER').strip()
        EMAIL_HOST_PASSWORD = env('EMAIL_HOST_PASSWORD').strip()
        EMAIL_HOST = env('EMAIL_HOST').strip()
        EMAIL_USE_TLS = True
        EMAIL_PORT = 25
    elif EMAIL_SERVER_TYPE == 'external':
        EMAIL_HOST_USER = env('EMAIL_HOST_USER').strip()
        EMAIL_HOST_PASSWORD = env('EMAIL_HOST_PASSWORD').strip()
        EMAIL_HOST = env('EMAIL_HOST').strip()
        EMAIL_USE_TLS = True #default
        EMAIL_PORT = 587 #default
    elif EMAIL_SERVER_TYPE == 'console':
        DEBUG=True
    else: 
        #custom setup, you do you :)
        EMAIL_HOST_USER = env('EMAIL_HOST_USER').strip()
        EMAIL_HOST_PASSWORD = env('EMAIL_HOST_PASSWORD').strip()
        EMAIL_HOST = env('EMAIL_HOST').strip()
        EMAIL_USE_TLS = True
        EMAIL_PORT = 25 
        logger.warning(f'Email configuration - Custom SMTP {EMAIL_HOST_USER}')
        if EMAIL_HOST_USER == '':
            logger.critical(f"Your seem to have forgotten to configure the email information.")
            logger.info(f"Your 'EMAIL_SERVER_TYPE' is set to '{EMAIL_SERVER_TYPE}'.\nThis seems to be is misconfigured.")
            raise SystemExit(logger.warning(f"Please check your {os.path.basename(__file__)} file"))

#### Local / debug ####
if (DEBUG == True) or (EMAIL_ACTIVATE_SENDING == False):
    logger.error("Emails will not send, only print in the console")
    EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"
    EMAIL_HOST = 'localhost'
    EMAIL_PORT = 1025
    EMAIL_HOST_USER = ''
    EMAIL_HOST_PASSWORD = ''
    EMAIL_USE_TLS = False
    DEFAULT_FROM_EMAIL = 'testing@example.com'


####################################################
##########  Registration
####################################################
ACCOUNT_ACTIVATION_DAYS = 3  # days
REGISTRATION_FIELDS = ('first_name', 'last_name', 'email', 'username', 'password1', 'password2')
EMAIL_ON_ACCOUNT_CREATION = False

####################################################
##########  Admin Plateform - Activate logs
####################################################
DJANGO_ADMIN_LOGS_ENABLED = True

DATA_UPLOAD_MAX_NUMBER_FIELDS=3000

####################################################
########## Django-q 
####################################################
Q_CLUSTER = {
    'name': 'server',
    'workers': 2,  # Number of workers, adjust this as needed
    'timeout': 90,
    'retry': 1,
    # 'queue_limit': 50,
    # 'bulk': 10,
    'orm': 'default',  # Use Django's ORM as the broker
}